from django.conf.urls.defaults import patterns, url

urlpatterns = patterns(
    '',
    url(r'^upload/', 'upy.contrib.ckeditor.views.upload', name='ckeditor_upload'),
    url(r'^browse/', 'upy.contrib.ckeditor.views.browse', name='ckeditor_browse'),
)



from django.conf import settings
if 'upy.contrib.g11n' in settings.INSTALLED_APPS:
    try:
        from upy.contrib.g11n.management.commands import g11n_validate
        c = g11n_validate.Command()
        c.execute()
    except Exception, e:
        print "\nError in upy.contrib.g11n.__init__: ",e


"""
You can find complete documentation to: `cke documentation`_

.. _cke documentation: https://github.com/dwaiter/django-ckeditor
"""
import os

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

if 'ckeditor' in settings.INSTALLED_APPS:
    # Confirm CKEDITOR_MEDIA_PREFIX setting has been specified.
    try:                
        settings.CKEDITOR_MEDIA_PREFIX
    except AttributeError:
        raise ImproperlyConfigured("django-ckeditor requires CKEDITOR_MEDIA_PREFIX setting. This setting specifies a URL prefix to the ckeditor JS and CSS media (not uploaded media). Make sure to use a trailing slash: CKEDITOR_MEDIA_PREFIX = '/media/ckeditor/'")
    
    # Confirm CKEDITOR_UPLOAD_PATH setting has been specified.
    try:                
        settings.CKEDITOR_UPLOAD_PATH
    except AttributeError:
        raise ImproperlyConfigured("django-ckeditor requires CKEDITOR_UPLOAD_PATH setting. This setting specifies an absolute path to your ckeditor media upload directory. Make sure you have write permissions for the path, i.e.: CKEDITOR_UPLOAD_PATH = '/home/media/media.lawrence.com/uploads'")

    # If a CKEDITOR_UPLOAD_PATH settings has been specified, confirm it exists.
    if getattr(settings, 'CKEDITOR_UPLOAD_PATH', None):
        if not os.path.exists(settings.CKEDITOR_UPLOAD_PATH):
            raise ImproperlyConfigured("django-ckeditor CKEDITOR_UPLOAD_PATH setting error, no such file or directory: '%s'" % settings.CKEDITOR_UPLOAD_PATH)



from django.template import Library
from upy.contrib.customadmin.models import CustomApp,CustomModel,CustomAdmin

register = Library()

@register.filter
def add_app_icons(app_list, autocomplete):
    """
    It returns a list of applications to view in admin's interface (index.html). 
    This filter is called only if you want view icons mode.
    """
    try:
        customadmin = CustomAdmin.objects.get(is_default="default")
        def_app_img = None
        if customadmin.default_app_image:
            def_app_img = customadmin.app_image
    except CustomAdmin.DoesNotExist:
        def_app_img = None
    app_icon_list = CustomApp.objects.select_related().all()
    app_list_ok = []
    for appicon in app_icon_list:
        try:
            app_temp = [x for x in app_list if x.get('name').lower() == appicon.application.lower()][0]
            app_temp['image'] = def_app_img
            if appicon.image:
                app_temp['image'] = appicon.image
            app_temp['verbose_app_name'] = appicon.verbose_app_name
            app_list_ok.append(app_temp)
            #app_list_ok.extend([x for x in app_list if x.get('name') == appicon.application])
        except IndexError, e:
            print ("Warning! In customadmin.templatetags.add_app_icons: ", 
                   e, "; application: ", appicon.application)
            pass # probabilmente e' stata inserita nella lista di applicazioni con icona 
                # una applicazione che non e' tra quelle registrate nell'admin, 
                # quindi poco male...
      
    if autocomplete:  
        app_list_ok_name = [x.get('name') for x in app_list_ok]
        app_list_name = [x.get('name') for x in app_list]
        app_out_name = list(set(app_list_name)-set(app_list_ok_name)) 
        app_out_list = [x for x in app_list if x.get('name') in app_out_name]
        
        for app_out in app_out_list:
            app_out['image'] = def_app_img
            app_out['verbose_app_name'] = app_out.get('name')
        
        app_list_ok.extend(app_out_list)
        
    return app_list_ok


@register.filter
def custom_app_list(app_list, autocomplete):
    """
    It returns a list of applications chosen to view in admin's interface (index.html).
    """
    app_icon_list = CustomApp.objects.select_related().all()
    app_list_ok = []
    for appicon in app_icon_list:
        try:
            app_temp = [x for x in app_list if x.get('name').lower() == appicon.application.lower()][0]
            app_temp['show_models'] = appicon.show_models
            app_temp['verbose_app_name'] = appicon.verbose_app_name
            app_list_ok.append(app_temp)
            #app_list_ok.extend([x for x in app_list if x.get('name') == appicon.application])
        except IndexError:
            #print ("Error in customadmin.templatetags.add_app_icons: ", e, 
            #       "; application: ", appicon.application)
            pass # probabilmente e' stata inserita nella lista di applicazioni con icona una applicazione
            # che non e' tra quelle registrate nell'admin, quindi poco male...
    
    if autocomplete:
        app_list_ok_name = [x.get('name') for x in app_list_ok]
        app_list_name = [x.get('name') for x in app_list]
        app_out_name = list(set(app_list_name)-set(app_list_ok_name)) 
        app_out_list = [x for x in app_list if x.get('name') in app_out_name]
        
        for app_out in app_out_list:
            app_out['show_models'] = True
            app_out['verbose_app_name'] = app_out.get('name')
    
        app_list_ok.extend(app_out_list)
       
    return app_list_ok

@register.filter
def add_model_icons(app_list,custom_admin):
    """
    It returns a list of models to view in admin's interface (index.html). 
    This filter is called only if you want view icons mode for models.
    """
    try:
        customadmin = CustomAdmin.objects.get(is_default="default")
        def_model_img = None
        if customadmin.default_model_image:
            def_model_img = customadmin.model_image
    except CustomAdmin.DoesNotExist:
        def_model_img = None
    custommodels = CustomModel.objects.select_related().all()
    app_res = []
    for app in app_list:
        names = []
        models_temp = []
        models_list = app['models']
        
        for m in custommodels:
            names.append(m.model.lower())
            try:
                model_temp = [x for x in models_list if u"%s" % x['name'].lower() == u"%s" % m.model.lower()][0]
                model_temp['image'] = def_model_img
                if m.image:
                    model_temp['image'] = m.image
                models_temp.append(model_temp)
            except IndexError:
                pass
        
        if custom_admin.autocomplete_models_list:
            for m in models_list:
                if m['name'].lower() not in names:
                    m['image'] = def_model_img
                    models_temp.append(m)
        

        if models_temp:
            app['models'] = models_temp
            app_res.append(app)
            
    return app_res


"""
TODO
"""
from django.views.decorators.csrf import csrf_exempt


@csrf_exempt
def img_preview(request):
    pass
    """
    if request.is_ajax():
        id = request.POST.get("id_img")
        try:
            image_mc = UPYImageMetaContent.g11nFilter.all_current(request, image__pk = id)[0] 
        except:
            dict = {"status": "ko",}
        else:
            dict = {"status": "ok", "url_thumb": image_mc.image.thumbnail_admin.url, "title": image_mc.title, "alt": image_mc.alt}
        finally:
            data = simplejson.dumps(dict)
            return HttpResponse(data)
    """

def get_img_preview():
    pass
    """
    def img_preview(fk_image):
        img = fk_image.preview_img
        return '<img src="%s" />' % img.admin_thumbnail.url
    img_preview.short_description = "Image Preview"
    img_preview.allow_tags = True
    return img_preview
    """


"""
This views are defined only for examples. They doesn't work.
"""
from django.http import HttpResponse
from upy.contrib.newsletter.utility import UPYNewsletterForm,upy_nl_confirm 
from upy.contrib.tree.views import upy_render
from django.utils.translation import ugettext_lazy as _
from django.utils import simplejson
from django.conf import settings

def send_confirm_mail(request):
    """
    It's used to send a confirmation mail
    """
    form = UPYNewsletterForm(True,request.POST)
    status = form.send_confirm_mail(request)
    data = simplejson.dumps({"status":status})
    return HttpResponse(data)


def confirm_subscription(request, secret_key):
    """
    When you want create a view like this, to serve a confirmation link, remeberd that 
    it's need an url decorated with a regex exactly like this following regex: (?P<secret_key>.+)
    """
    rendered = upy_nl_confirm(request,secret_key,message_ok = _(u"Hi %s, your account is succesfully confirmed"), 
                         message_error = _(u"Some problems are occurred during confirmation. Please, contact %s"),
                         use_contact = True, email_host = settings.EMAIL_HOST_USER)
    return HttpResponse(rendered)
    
    
    
    
def newsletter(request,template_name):
    """
    It's used to create a newsletter form to render in a template
    """
    form = UPYNewsletterForm(True)
    return upy_render(request, template_name,{"form":form})



from django import forms
from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.safestring import mark_safe
from django.utils.html import conditional_escape
from django.utils.encoding import force_unicode
from django.utils import simplejson

from django.core.exceptions import ImproperlyConfigured
from django.forms.util import flatatt

json_encode = simplejson.JSONEncoder().encode

DEFAULT_CONFIG = {
    'skin': 'office2003',
    'toolbar': 'Full',
    'height': 300,
    'width': 800,
    #'contentsCss':'/static/example_layout.css'
}

class CKEditorWidget(forms.Textarea):
    """
    Widget providing CKEditor for Rich Text Editing.
    Supports direct image uploads and embed.
    """
    class Media:
        try:
            js = (
                settings.CKEDITOR_MEDIA_PREFIX + 'ckeditor/ckeditor.js',
            )
        except AttributeError:
            raise ImproperlyConfigured("django-ckeditor requires CKEDITOR_MEDIA_PREFIX setting. This setting specifies a URL prefix to the ckeditor JS and CSS media (not uploaded media). Make sure to use a trailing slash: CKEDITOR_MEDIA_PREFIX = '/media/ckeditor/'")

    def __init__(self, config_name='default',config = None, *args, **kwargs):
        super(CKEditorWidget, self).__init__(*args, **kwargs)
        # Setup config from defaults.
        if not config:
            self.config = DEFAULT_CONFIG
        else:
            self.config = config
        if not self.config.has_key('skin'):
            self.config['skin'] = DEFAULT_CONFIG['skin']
        if not self.config.has_key('toolbar'):
            self.config['toolbar'] = DEFAULT_CONFIG['toolbar']
            
    def render(self, name, value, attrs={}):
        if value is None: value = ''
        final_attrs = self.build_attrs(attrs, name=name)
        self.config['filebrowserUploadUrl'] = reverse('ckeditor_upload')
        self.config['filebrowserBrowseUrl'] = reverse('ckeditor_browse')
        return mark_safe(u'''<textarea%s>%s</textarea>
        <script type="text/javascript">
            CKEDITOR.replace("%s", %s);
        </script>''' % (flatatt(final_attrs), conditional_escape(force_unicode(value)), final_attrs['id'], json_encode(self.config)))



"""
It defines the urlpattern for css file with rules for admin's interface customization.
"""
from django.conf.urls.defaults import *
urlpatterns = patterns('',
        (r'^upy_custom_admin_layout.css$','upy.contrib.customadmin.views.custom_admin_layout'),
)



from django.conf.urls.defaults import *
from django.conf import settings
from upy.contrib.tree.utility import getUrlList



urlpatterns = patterns('',
        # UPY URLS CONFIGURATION
        (r'^favicon.ico$', 'upy.contrib.tree.views.favicon'),
        (r'^sitemap.xml$', 'upy.contrib.tree.views.sitemap'),
        (r'^robots.txt$', 'upy.contrib.tree.views.robots'),
        (r'^upy_get_languages$','upy.contrib.tree.views.get_languages'),  
)
upy_urls, urls_login_required, tree_urls = getUrlList()

for app_url in upy_urls:
    urlpatterns += patterns('', app_url)
if settings.USE_UPY_ROUTING:
    urlpatterns += patterns('',
        (r'^.*$', 'upy.contrib.tree.views.render_page'),
    )

LOGIN_REQUIRED_URLS = urls_login_required
TREE_URLS = tree_urls


"""
Contains some common upy project context_processors
"""
from django.conf import settings
import upy

def use_upy_admin(request):
    """
    Adds settings.USE_UPY_ADMIN and settings.JQUERY_LIB to context dictionary
    """
    context_extras = {}
    context_extras['USE_UPY_ADMIN'] = settings.USE_UPY_ADMIN
    context_extras['JQUERY_LIB'] = settings.JQUERY_LIB
    context_extras['USE_UPY_JQUERY_LIB'] = settings.USE_UPY_JQUERY_LIB
    context_extras['USE_UPY_CSS_RESET'] = settings.USE_UPY_CSS_RESET
    context_extras['USE_UPY_ROSETTA'] = settings.USE_UPY_ROSETTA
    context_extras['UPY_VERSION'] = upy.__version__
    return context_extras


import os
from datetime import datetime

from django.conf import settings
from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext
            
try: 
    from PIL import Image, ImageOps 
except ImportError: 
    import Image, ImageOps

try:
    from django.views.decorators.csrf import csrf_exempt
except ImportError:
    # monkey patch this with a dummy decorator which just returns the same function
    # (for compatability with pre-1.1 Djangos)
    def csrf_exempt(fn):
        return fn
        
THUMBNAIL_SIZE = (75, 75)
    
def get_available_name(name):
    """
    Returns a filename that's free on the target storage system, and
    available for new content to be written to.
    """
    dir_name, file_name = os.path.split(name)
    file_root, file_ext = os.path.splitext(file_name)
    # If the filename already exists, keep adding an underscore (before the
    # file extension, if one exists) to the filename until the generated
    # filename doesn't exist.
    while os.path.exists(name):
        file_root += '_'
        # file_ext includes the dot.
        name = os.path.join(dir_name, file_root + file_ext)
    return name

def get_thumb_filename(file_name):
    """
    Generate thumb filename by adding _thumb to end of filename before . (if present)
    """
    return '%s_thumb%s' % os.path.splitext(file_name)

def create_thumbnail(filename):
    image = Image.open(filename)
        
    # Convert to RGB if necessary
    # Thanks to Limodou on DjangoSnippets.org
    # http://www.djangosnippets.org/snippets/20/
    if image.mode not in ('L', 'RGB'):
        image = image.convert('RGB')
       
    # scale and crop to thumbnail
    imagefit = ImageOps.fit(image, THUMBNAIL_SIZE, Image.ANTIALIAS)
    imagefit.save(get_thumb_filename(filename))
        
def get_media_url(path):
    """
    Determine system file's media URL.
    """
    upload_prefix = getattr(settings, "CKEDITOR_UPLOAD_PREFIX", None)
    if upload_prefix:
        url = upload_prefix + path.replace(settings.CKEDITOR_UPLOAD_PATH, '')
    else:
        url = settings.MEDIA_URL + path.replace(settings.MEDIA_ROOT, '')
   
    # Remove any double slashes.
    return url.replace('//', '/')

def get_upload_filename(upload_name, user):
    # If CKEDITOR_RESTRICT_BY_USER is True upload file to user specific path.
    if getattr(settings, 'CKEDITOR_RESTRICT_BY_USER', False):
        user_path = user.username
    else:
        user_path = ''

    # Generate date based path to put uploaded file.
    date_path = datetime.now().strftime('%Y/%m/%d')
    
    # Complete upload path (upload_path + date_path).
    upload_path = os.path.join(settings.CKEDITOR_UPLOAD_PATH, user_path, date_path)
   
    # Make sure upload_path exists.
    if not os.path.exists(upload_path):
        os.makedirs(upload_path)
    
    # Get available name and return.
    return get_available_name(os.path.join(upload_path, upload_name))
     
    
@csrf_exempt
def upload(request):
    """
    Uploads a file and send back its URL to CKEditor.

    TODO:
        Validate uploads
    """
    # Get the uploaded file from request.
    upload = request.FILES['upload']
    upload_ext = os.path.splitext(upload.name)[1]
   
    # Open output file in which to store upload. 
    upload_filename = get_upload_filename(upload.name, request.user)
    out = open(upload_filename, 'wb+')

    # Iterate through chunks and write to destination.
    for chunk in upload.chunks():
        out.write(chunk)
    out.close()

    create_thumbnail(upload_filename)

    # Respond with Javascript sending ckeditor upload url.
    url = get_media_url(upload_filename)
    return HttpResponse("""
    <script type='text/javascript'>
        window.parent.CKEDITOR.tools.callFunction(%s, '%s');
    </script>""" % (request.GET['CKEditorFuncNum'], url))

def get_image_browse_urls(user=None):
    """
    Recursively walks all dirs under upload dir and generates a list of
    thumbnail and full image URL's for each file found.
    """
    images = []
    
    # If a user is provided and CKEDITOR_RESTRICT_BY_USER is True,
    # limit images to user specific path, but not for superusers.
    if user and not user.is_superuser and getattr(settings, 'CKEDITOR_RESTRICT_BY_USER', False):
        user_path = user.username
    else:
        user_path = ''

    browse_path = os.path.join(settings.CKEDITOR_UPLOAD_PATH, user_path)
    
    for root, dirs, files in os.walk(browse_path):
        for filename in [ os.path.join(root, x) for x in files ]:
            # bypass for thumbs
            if '_thumb' in filename:
                continue
            
            images.append({
                'thumb': get_media_url(get_thumb_filename(filename)),
                'src': get_media_url(filename)
            })

    return images
    
def browse(request):
    context = RequestContext(request, {
        'images': get_image_browse_urls(request.user),
        'media_prefix': settings.CKEDITOR_MEDIA_PREFIX,
    })
    return render_to_response('browse.html', context)



"""
Contains some fields as utilities.
"""
from django.utils.translation import ugettext as _
from django.db import models


# ISO 3166-1 country names and codes adapted from http://opencountrycodes.appspot.com/python/
COUNTRIES = (
    ('GB', _('United Kingdom')), 
    ('AF', _('Afghanistan')), 
    ('AX', _('Aland Islands')), 
    ('AL', _('Albania')), 
    ('DZ', _('Algeria')), 
    ('AS', _('American Samoa')), 
    ('AD', _('Andorra')), 
    ('AO', _('Angola')), 
    ('AI', _('Anguilla')), 
    ('AQ', _('Antarctica')), 
    ('AG', _('Antigua and Barbuda')), 
    ('AR', _('Argentina')), 
    ('AM', _('Armenia')), 
    ('AW', _('Aruba')), 
    ('AU', _('Australia')), 
    ('AT', _('Austria')), 
    ('AZ', _('Azerbaijan')), 
    ('BS', _('Bahamas')), 
    ('BH', _('Bahrain')), 
    ('BD', _('Bangladesh')), 
    ('BB', _('Barbados')), 
    ('BY', _('Belarus')), 
    ('BE', _('Belgium')), 
    ('BZ', _('Belize')), 
    ('BJ', _('Benin')), 
    ('BM', _('Bermuda')), 
    ('BT', _('Bhutan')), 
    ('BO', _('Bolivia')), 
    ('BA', _('Bosnia and Herzegovina')), 
    ('BW', _('Botswana')), 
    ('BV', _('Bouvet Island')), 
    ('BR', _('Brazil')), 
    ('IO', _('British Indian Ocean Territory')), 
    ('BN', _('Brunei Darussalam')), 
    ('BG', _('Bulgaria')), 
    ('BF', _('Burkina Faso')), 
    ('BI', _('Burundi')), 
    ('KH', _('Cambodia')), 
    ('CM', _('Cameroon')), 
    ('CA', _('Canada')), 
    ('CV', _('Cape Verde')), 
    ('KY', _('Cayman Islands')), 
    ('CF', _('Central African Republic')), 
    ('TD', _('Chad')), 
    ('CL', _('Chile')), 
    ('CN', _('China')), 
    ('CX', _('Christmas Island')), 
    ('CC', _('Cocos (Keeling) Islands')), 
    ('CO', _('Colombia')), 
    ('KM', _('Comoros')), 
    ('CG', _('Congo')), 
    ('CD', _('Congo, The Democratic Republic of the')), 
    ('CK', _('Cook Islands')), 
    ('CR', _('Costa Rica')), 
    ('CI', _('Cote d\'Ivoire')), 
    ('HR', _('Croatia')), 
    ('CU', _('Cuba')), 
    ('CY', _('Cyprus')), 
    ('CZ', _('Czech Republic')), 
    ('DK', _('Denmark')), 
    ('DJ', _('Djibouti')), 
    ('DM', _('Dominica')), 
    ('DO', _('Dominican Republic')), 
    ('EC', _('Ecuador')), 
    ('EG', _('Egypt')), 
    ('SV', _('El Salvador')), 
    ('GQ', _('Equatorial Guinea')), 
    ('ER', _('Eritrea')), 
    ('EE', _('Estonia')), 
    ('ET', _('Ethiopia')), 
    ('FK', _('Falkland Islands (Malvinas)')), 
    ('FO', _('Faroe Islands')), 
    ('FJ', _('Fiji')), 
    ('FI', _('Finland')), 
    ('FR', _('France')), 
    ('GF', _('French Guiana')), 
    ('PF', _('French Polynesia')), 
    ('TF', _('French Southern Territories')), 
    ('GA', _('Gabon')), 
    ('GM', _('Gambia')), 
    ('GE', _('Georgia')), 
    ('DE', _('Germany')), 
    ('GH', _('Ghana')), 
    ('GI', _('Gibraltar')), 
    ('GR', _('Greece')), 
    ('GL', _('Greenland')), 
    ('GD', _('Grenada')), 
    ('GP', _('Guadeloupe')), 
    ('GU', _('Guam')), 
    ('GT', _('Guatemala')), 
    ('GG', _('Guernsey')), 
    ('GN', _('Guinea')), 
    ('GW', _('Guinea-Bissau')), 
    ('GY', _('Guyana')), 
    ('HT', _('Haiti')), 
    ('HM', _('Heard Island and McDonald Islands')), 
    ('VA', _('Holy See (Vatican City State)')), 
    ('HN', _('Honduras')), 
    ('HK', _('Hong Kong')), 
    ('HU', _('Hungary')), 
    ('IS', _('Iceland')), 
    ('IN', _('India')), 
    ('ID', _('Indonesia')), 
    ('IR', _('Iran, Islamic Republic of')), 
    ('IQ', _('Iraq')), 
    ('IE', _('Ireland')), 
    ('IM', _('Isle of Man')), 
    ('IL', _('Israel')), 
    ('IT', _('Italy')), 
    ('JM', _('Jamaica')), 
    ('JP', _('Japan')), 
    ('JE', _('Jersey')), 
    ('JO', _('Jordan')), 
    ('KZ', _('Kazakhstan')), 
    ('KE', _('Kenya')), 
    ('KI', _('Kiribati')), 
    ('KP', _('Korea, Democratic People\'s Republic of')), 
    ('KR', _('Korea, Republic of')), 
    ('KW', _('Kuwait')), 
    ('KG', _('Kyrgyzstan')), 
    ('LA', _('Lao People\'s Democratic Republic')), 
    ('LV', _('Latvia')), 
    ('LB', _('Lebanon')), 
    ('LS', _('Lesotho')), 
    ('LR', _('Liberia')), 
    ('LY', _('Libyan Arab Jamahiriya')), 
    ('LI', _('Liechtenstein')), 
    ('LT', _('Lithuania')), 
    ('LU', _('Luxembourg')), 
    ('MO', _('Macao')), 
    ('MK', _('Macedonia, The Former Yugoslav Republic of')), 
    ('MG', _('Madagascar')), 
    ('MW', _('Malawi')), 
    ('MY', _('Malaysia')), 
    ('MV', _('Maldives')), 
    ('ML', _('Mali')), 
    ('MT', _('Malta')), 
    ('MH', _('Marshall Islands')), 
    ('MQ', _('Martinique')), 
    ('MR', _('Mauritania')), 
    ('MU', _('Mauritius')), 
    ('YT', _('Mayotte')), 
    ('MX', _('Mexico')), 
    ('FM', _('Micronesia, Federated States of')), 
    ('MD', _('Moldova')), 
    ('MC', _('Monaco')), 
    ('MN', _('Mongolia')), 
    ('ME', _('Montenegro')), 
    ('MS', _('Montserrat')), 
    ('MA', _('Morocco')), 
    ('MZ', _('Mozambique')), 
    ('MM', _('Myanmar')), 
    ('NA', _('Namibia')), 
    ('NR', _('Nauru')), 
    ('NP', _('Nepal')), 
    ('NL', _('Netherlands')), 
    ('AN', _('Netherlands Antilles')), 
    ('NC', _('New Caledonia')), 
    ('NZ', _('New Zealand')), 
    ('NI', _('Nicaragua')), 
    ('NE', _('Niger')), 
    ('NG', _('Nigeria')), 
    ('NU', _('Niue')), 
    ('NF', _('Norfolk Island')), 
    ('MP', _('Northern Mariana Islands')), 
    ('NO', _('Norway')), 
    ('OM', _('Oman')), 
    ('PK', _('Pakistan')), 
    ('PW', _('Palau')), 
    ('PS', _('Palestinian Territory, Occupied')), 
    ('PA', _('Panama')), 
    ('PG', _('Papua New Guinea')), 
    ('PY', _('Paraguay')), 
    ('PE', _('Peru')), 
    ('PH', _('Philippines')), 
    ('PN', _('Pitcairn')), 
    ('PL', _('Poland')), 
    ('PT', _('Portugal')), 
    ('PR', _('Puerto Rico')), 
    ('QA', _('Qatar')), 
    ('RE', _('Reunion')), 
    ('RO', _('Romania')), 
    ('RU', _('Russian Federation')), 
    ('RW', _('Rwanda')), 
    ('BL', _('Saint Barthelemy')), 
    ('SH', _('Saint Helena')), 
    ('KN', _('Saint Kitts and Nevis')), 
    ('LC', _('Saint Lucia')), 
    ('MF', _('Saint Martin')), 
    ('PM', _('Saint Pierre and Miquelon')), 
    ('VC', _('Saint Vincent and the Grenadines')), 
    ('WS', _('Samoa')), 
    ('SM', _('San Marino')), 
    ('ST', _('Sao Tome and Principe')), 
    ('SA', _('Saudi Arabia')), 
    ('SN', _('Senegal')), 
    ('RS', _('Serbia')), 
    ('SC', _('Seychelles')), 
    ('SL', _('Sierra Leone')), 
    ('SG', _('Singapore')), 
    ('SK', _('Slovakia')), 
    ('SI', _('Slovenia')), 
    ('SB', _('Solomon Islands')), 
    ('SO', _('Somalia')), 
    ('ZA', _('South Africa')), 
    ('GS', _('South Georgia and the South Sandwich Islands')), 
    ('ES', _('Spain')), 
    ('LK', _('Sri Lanka')), 
    ('SD', _('Sudan')), 
    ('SR', _('Suriname')), 
    ('SJ', _('Svalbard and Jan Mayen')), 
    ('SZ', _('Swaziland')), 
    ('SE', _('Sweden')), 
    ('CH', _('Switzerland')), 
    ('SY', _('Syrian Arab Republic')), 
    ('TW', _('Taiwan, Province of China')), 
    ('TJ', _('Tajikistan')), 
    ('TZ', _('Tanzania, United Republic of')), 
    ('TH', _('Thailand')), 
    ('TL', _('Timor-Leste')), 
    ('TG', _('Togo')), 
    ('TK', _('Tokelau')), 
    ('TO', _('Tonga')), 
    ('TT', _('Trinidad and Tobago')), 
    ('TN', _('Tunisia')), 
    ('TR', _('Turkey')), 
    ('TM', _('Turkmenistan')), 
    ('TC', _('Turks and Caicos Islands')), 
    ('TV', _('Tuvalu')), 
    ('UG', _('Uganda')), 
    ('UA', _('Ukraine')), 
    ('AE', _('United Arab Emirates')), 
    ('US', _('United States')), 
    ('UM', _('United States Minor Outlying Islands')), 
    ('UY', _('Uruguay')), 
    ('UZ', _('Uzbekistan')), 
    ('VU', _('Vanuatu')), 
    ('VE', _('Venezuela')), 
    ('VN', _('Viet Nam')), 
    ('VG', _('Virgin Islands, British')), 
    ('VI', _('Virgin Islands, U.S.')), 
    ('WF', _('Wallis and Futuna')), 
    ('EH', _('Western Sahara')), 
    ('YE', _('Yemen')), 
    ('ZM', _('Zambia')), 
    ('ZW', _('Zimbabwe')), 
)

class CountryField(models.CharField):
    """
    Is a CharField with the complete list of countries as choices.
    """
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('max_length', 2)
        kwargs.setdefault('choices', COUNTRIES)

        super(CountryField, self).__init__(*args, **kwargs)

    def get_internal_type(self):
        """
        Returns internal type of this field: CharField as string
        """
        return "CharField"
        
try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^upy\.fields\.CountryField"])
except ImportError:
    pass


from django.utils.translation import ugettext_lazy as _
from django.db import models
from imagekit.models import ImageSpecField, ProcessedImageField
from imagekit.processors import ResizeToFit, SmartResize, Adjust
from datetime import date
from django.conf import settings
from upy.contrib.g11n.models import G11nModel

def _choose_preprocessor():
    """
    Function to define the preprocessor on original_image field. It crops the original image, so user
    cannot save on server too big images. (in order to prevent disk space wasting)
    """
    if settings.USE_FULLHD_SUPPORT:
        return ResizeToFit(width=1920, height=1366) #upscale=False)
    else:
        return ResizeToFit(width=800, height=800) #upscale=False)

    
class UPYImage(models.Model):
    """
    This abstract class only provide original_image and admin_thumbnail field, with the 
    admin_thumbnail_view function. Inherit this class in you models.py and import imagekit ImageSpec
    to create other thumbnail or display field with any processor you need.
    """
    original_image = ProcessedImageField(null = True, blank = True, upload_to='upyimage', processors=[_choose_preprocessor(),])
    admin_thumbnail = ImageSpecField([Adjust(contrast=1.2, sharpness=1.1),
            SmartResize(25, 25)], image_field='original_image', options={'quality': 90})  #format='JPEG', 

    def admin_thumbnail_view(self):
        """
        It's a property used in admin interface. It decorates list_display items with a thumbnail 
        and with prettyPhoto jQuery plugin to open images in a lightbox.
        """
        if self.original_image:
            return u'<a href="%s" data-rel="prettyPhoto" title="%s"><img src="%s" class="upy_admin_thumb" alt="%s"/></a>' % (self.original_image.url, _("view image"), self.admin_thumbnail.url, self.admin_thumbnail.url )
        else:
            return None
    admin_thumbnail_view.short_description = _(u'Thumbnail')
    admin_thumbnail_view.allow_tags = True
    
    original_image_label = _(u"Original image")
    
    def __unicode__(self):
        return u"%s" % (self.original_image)

    class Meta:
        abstract = True
                
class PositionImage(UPYImage):
    """
    Class for Images with position. It adds position field and manages position
    on admin interface.
    """
    position = models.PositiveSmallIntegerField(u'Position', default=0)

    class Meta: 
        ordering = [u'position',]
        abstract = True   
            
            
if settings.USE_UPY_G11N:
        
    class UPYImageG11n(G11nModel):
        """
        Abstract class for G11nModel to inherit if you need localized contents for your UPYImage model
        """
        title = models.CharField(max_length = 150, help_text = _(u"Set the image title."))
        alt = models.CharField(max_length = 150, help_text = _(u"Set the image alternative field."))
        
        def __unicode__(self):
            return u"%s" % (self.language)
        
        class Meta:
            abstract = True

if settings.USE_UPY_IMAGE:            
    class ImageRepositoryBase(PositionImage):
        """
        Concrete class for Images. It add useful fields like position, name, date... and manage position
        on the admin.
        """
        name = models.CharField(max_length=100)
        display_image = ImageSpecField([ResizeToFit(800,600)], image_field='original_image', options={'quality': 90})  #format='JPEG',
        date = models.DateField(default = date.today, help_text = _(u"Set the date you want to display for this image.")) 
        creation_date = models.DateTimeField(auto_now_add = True, help_text = _(u"Establishment date"), 
                                             verbose_name = _(u"Creation date"))
        def admin_thumbnail_view(self):
            """
            It's a property used in admin interface. It decorates list_display items with a thumbnail 
            and with prettyPhoto jQuery plugin to open images in a lightbox.
            """
            if self.original_image:
                return u'<a href="%s" data-rel="prettyPhoto" title="%s"><img src="%s" alt="%s"/></a>' % (self.display_image.url, _("view image"), self.admin_thumbnail.url, self.admin_thumbnail.url )
            else:
                return None
        admin_thumbnail_view.short_description = _(u'Thumbnail Display')
        admin_thumbnail_view.allow_tags = True
        
        def get_name(self):
            return self.name
        
        def __unicode__(self):
            return u"%s" % (self.name)
            
        class Meta:
            verbose_name = _(u"Image")
            verbose_name_plural = _(u"Images")
            ordering = [u'position',]
            abstract = True
    
        
    if settings.USE_UPY_G11N:
        
                
        class ImageRepositoryG11n(UPYImageG11n):
            """
            It defines image's repository g11n model if G11n is used in this project
            """
            description = models.TextField(null = True, blank = True, help_text = _(u"Image description."))
            image_repository = models.ForeignKey(u"ImageRepository", help_text = _(u"Choose an Image Repository element to associate."), verbose_name = _(u"Image Repository element"))
    
        class ImageRepository(ImageRepositoryBase):
            """
            It defines image's repository model if G11n is used in this project
            """
            class G11nMeta:
                g11n = 'ImageRepositoryG11n'
                fieldname = 'image_repository'
    
    else:
        
        class ImageRepository(ImageRepositoryBase):
            """
            It defines image's repository model if G11n is not used in this project
            """
            title = models.CharField(max_length = 150, help_text = _(u"Set the image title."))
            alt = models.CharField(max_length = 150, help_text = _(u"Set the image alternative field."))
            description = models.TextField(null = True, blank = True, help_text = _(u"Image description."))


"""
It includes urlpatterns necessary for upy.contrib.images
"""
from django.conf.urls.defaults import *
urlpatterns = patterns('',
        (r'^img_preview$','upy.contrib.image.views.img_preview'),
)



from django.conf.urls.defaults import *
from django.conf import settings
#import colors

urlpatterns = patterns('',

    url(r'^$', 'django.views.generic.simple.direct_to_template', {
        'template': 'colors-test.html',
        'extra_context': {
            'colors_tests': [
                '000000', # Black
                'ff0000', # Red
                '008000', # Green 
                '008080', # teal
                '808000', # Olive
                '00ff00', # Lime
                '0000ff', # Blue
                'ffff00', # Yellow
                '00ffff', # Aqua / cyan
                'ff00ff', # Fuchsia / magenta
                '808080', # Gray
                'ffffff', # White
                '000080', # Navy
                'C0C0C0', # Silver
                '800080', # Purple
                '800000', # Maroon
            ],
            'lightness_range':  range(0, 101, 10),
            'saturation_range': range(0, 101, 10),
            'hue_range':        range(0, 361, 30),
            'version':          "0.0.2",#colors.get_version(),
            },
    }, name='django-colors-test'),


)




#!/usr/bin/python
"""
Use this snippet like python command to run manage's functions with uWSGI
"""
import os,sys
from project import config

os.environ.update({'UWSGI_INI':config.UWSGI_INI,
                   'UWSGI_PYRUN':sys.argv[1],
                   'UWSGI_PYARGV':" ".join(sys.argv[2:])})
os.system(config.UWSGI_COMMAND)


from django.contrib import admin
from upy.contrib.newsletter.models import Dispatcher, Contact, Newsletter, List, Attachment
from django import forms
from django.utils.translation import ugettext_lazy as _
from datetime import datetime
from upy.contrib.ckeditor.widgets import CKEditorWidget

class SendDateForm(forms.ModelForm):
    """
    This form clean a dispatcher and set send_date parameter to now() if it doesn't exists
    """
    class Meta:
        model = Dispatcher

    def clean_send_date(self):
        send_date = self.cleaned_data['send_date']
        if not send_date:
            send_date = datetime.now()
            #raise forms.ValidationError(_("The date cannot be in the past!"))
        return send_date

class ContactOption(admin.ModelAdmin):
    """
    Admin's options for Contact model
    """
    list_display = ('email','name','surname','confirmed','subscribed')
    fieldsets = (('', {'fields': 
                       (('name', 'surname',),('email', 'secret_key',),('confirmed','subscribed'),),
        },),)
    list_filter = ('confirmed','subscribed')
    save_on_top = True
    search_fields = ('name','surname','email',)
    class Meta:
        model = Contact  
    
class ListOption(admin.ModelAdmin):
    """
    Admin's options for List model
    """
    list_display = ('name','priority','test','description')
    list_editable = ('priority','description','test')
    fieldsets = (('', {'fields': 
                       (('name', 'publication', 'priority','test'),('description',),('contacts',),),
        },),)
    list_filter = ('priority',)
    filter_horizontal = ('contacts',)
    save_on_top = True
    search_fields = ('name',)
    class Meta:
        model = List  
        
class AttachmentOption(admin.ModelAdmin):
    """
    Admin's options for Attachment model
    """
    list_display = ('name','attached_file')
    fieldsets = (('', {'fields': 
                       (('name', 'attached_file',),),
        },),)
    save_on_top = True
    search_fields = ('name',)
    class Meta:
        model = Attachment 
        
class NLForm(forms.ModelForm):
    """
    This form change default widget to textarea input with CKEditor widget
    """
    body_text = forms.CharField(label = _(u"Body text"),widget=forms.Textarea(attrs={'cols': 80, 'rows': 15}), required = False, help_text = _(u"Set body in text format"))
    body_html = forms.CharField(widget=CKEditorWidget(config={'height': 300,'width': 800,}))
    class Meta:
        model = Newsletter
       
class NewsletterOption(admin.ModelAdmin):
    """
    Admin's options for Newsletter model
    """
    list_display = ('subject','sent','body_text')
    fieldsets = (('', {'fields': 
                       (('subject', 'sent',),('body_text','body_html'),('attachments',)),
        },),)
    list_filter = ('sent',)
    form = NLForm
    save_on_top = True
    search_fields = ('subject',)
    class Meta:
        model = Newsletter
        
class DispatcherOption(admin.ModelAdmin):
    """
    Admin's options for Dispatcher model
    """
    list_display = ('newsletter','send_date', 'sent_date', 'contact_list', 'priority', 'status',)
    #list_editable = ('send_date',)
    fieldsets = (('', {'fields': 
                       (('contact_list', 'newsletter',),('last_mail_sent','status','send_date'),),
        },),)
    list_filter = ('status','user_create')
    save_on_top = True
    form = SendDateForm
    class Meta:
        model = Dispatcher
    def save_model(self, request, obj, form, change):
        obj.user_create = request.user
        obj.save() 
        
    actions = ['stop_selected','revoke_selected','restart_selected','continue_selected']
    
    def stop_selected(self, request, queryset):
        """
        Stop all processing selected dispatchers
        """
        queryset.update(status='stopped')
    stop_selected.short_description = _("Stop selected dispatchers")     
    
    def revoke_selected(self, request, queryset):
        """
        Revoke all selected dispatchers
        """
        queryset.update(status='deleted')
    revoke_selected.short_description = _("Revoke selected dispatchers") 
    
    def restart_selected(self, request, queryset):
        """
        Restart all selected dispatchers
        """
        for q in queryset:
            q.status = 'waiting'
            q.save()
    restart_selected.short_description = _("Restart selected dispatchers now") 
    
    def continue_selected(self, request, queryset):
        """
        Continue all selected dispatchers
        """
        queryset.update(status='processing')
    continue_selected.short_description = _("Continue selected dispatchers")   
            
admin.site.register(Contact, ContactOption)
admin.site.register(List, ListOption)
admin.site.register(Attachment, AttachmentOption)
admin.site.register(Newsletter, NewsletterOption)
admin.site.register(Dispatcher, DispatcherOption)


"""
Contains some classes to extend ModelAdmin, with some other functionalities
"""
from django.contrib import admin

class ButtonLink(object):
    """
    It defines a button that you can locate near the history button in change_form.html as a link
    """
    def __init__(self, short_description, link, css_id=None, css_class=None):
        """
        Use it to create a ButtonableLink. It takes following parameters:
        'short_description' is what you want display in this link,
        'link' is the value of this link,
        'css_id' is the id selector,
        'css_class' is the class selector
        """
        self.short_description = short_description
        self.link = link
        self.css_id = css_id
        self.css_class = css_class
        
class ButtonForm(object):
    """
    It defines a button that you can locate near the history button in change_form.html as a form
    """
    def __init__(self, name, submit_value, form_action, input_dict, form_method="post", css_class=None):
        """
        Use it to create a ButtonableForm. It takes following parameters:
        'name' is the form's name,
        'submit_value' is the button's value,
        'form_action' is the form's action,
        'form_method' is the form's method,
        'css_class' is the class selector
        """
        self.name = name
        self.submit_value = submit_value
        self.form_action = form_action
        self.form_method = form_method
        self.input_dict = input_dict  #{'name':'value',}
        self.css_class = css_class
        
class ButtonableModelAdmin(admin.ModelAdmin):
    """
    Use this admin only if you want apply custom button on the change_form template in admin panel.
    """
    buttons_link=[]
    buttons_form=[]
    def change_view(self, request, object_id, form_url = '',extra_context={}): 
        """
        It adds ButtonLinks and ButtonForms to extra_context used in the change_form template
        """
        extra_context['buttons_link']=self.buttons_link
        extra_context['buttons_form']=self.buttons_form
        extra_context['button_object_id']=object_id  
        return super(ButtonableModelAdmin, self).change_view(request, object_id, form_url,extra_context)
    
class HiddenModelAdmin(admin.ModelAdmin):
     def get_model_perms(self, *args, **kwargs):
        perms = admin.ModelAdmin.get_model_perms(self, *args, **kwargs)
        perms['list_hide'] = True
        return perms


from django.conf import settings
from project import config
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
if config.USE_UPY_G11N:
    from upy.contrib.g11n.admin import G11nAdmin,G11nTabularInlineAdmin
    from upy.contrib.image.models import UPYImageG11n 
    if config.USE_UPY_IMAGE:
        from upy.contrib.image.models import ImageRepositoryG11n
if config.USE_UPY_IMAGE:
        from upy.contrib.image.models import ImageRepository
from django import forms
import Image as PilImage
from django.forms.fields import ImageField


def clean_image(obj,field_name):       
    """
    Function for the Image Form to check the filesize, according to UPYIMAGE settings
    """
    img_temp = obj.cleaned_data[field_name]
    if img_temp:
        im = PilImage.open(img_temp.file)
        width, height = im.size
        if height*width > settings.UPYIMAGE_LIMIT_AREA: # 2073600 = fullHD 1920x1080px
            #raise forms.ValidationError(_("Image size out of range. Actual size: %(width)spx X %(height)spx" % {'width':width, 'height':height}))
            obj._errors[field_name] = obj.error_class([_("Image size out of range. Actual size: %(width)spx X %(height)spx" % {'width':width, 'height':height})])
            
        if settings.RGBA_FILTER:
            mode = im.mode
            if mode not in ("RGB","RGBA"):
                #raise forms.ValidationError(_("Image mode not in RGB or RGBa."))
                obj._errors[field_name] = obj.error_class([_("Image mode not in RGB or RGBa.")])


class UPYImageForm(forms.ModelForm):
    """
    Abstract form to use for UPYImage
    """ 
    def __init__(self,*args, **kwargs):
        super(UPYImageForm, self).__init__(*args, **kwargs)
        self.fields['original_image'].label = self.instance.original_image_label
              
    
    def clean(self):
        """
        It cleans all ImageField in this form
        """
        for name,field in self.fields.items():
            if issubclass(field.__class__,ImageField):
                clean_image(self,name)
        return self.cleaned_data
    
class UPYImageOption(admin.ModelAdmin):
    """
    Abstract admin option class for UPYImage
    """
    list_display = ('original_image','admin_thumbnail_view',)
    form = UPYImageForm
    
    class Media:
        css = {"all" : ("/upy_static/css/prettyPhoto.css",)}
        js = (settings.JQUERY_LIB,
              settings.JQUERYUI_LIB,
              '/upy_static/js/lib/jquery.prettyPhoto.js',
              '/upy_static/js/prettyPhoto-init.js')
        
    class Meta:
        abstract = True
    
    
class PositionImageOption(UPYImageOption):
    """
    Abstract admin option class for PositionImage
    """
    list_display = ('position','original_image','admin_thumbnail_view',)
    list_editable = ['position',]
    list_display_links = ['original_image',]
    ordering = ('position',)
    exclude = ('position',)
    
    class Media:
        js = UPYImageOption.Media.js+('/upy_static/js/admin-list-reorder.js',)
    
    class Meta:
        abstract = True
    
    

if config.USE_UPY_G11N:

    class UPYImageG11nInline(G11nTabularInlineAdmin):
        """
        Abstract admin option class for UPYImageG11n inline
        """
        fieldsets = (('', {'fields': 
                           ('title','alt'),
            },),) + G11nAdmin.fieldsets
        model = UPYImageG11n
        class Meta:
            abstract = True

if config.USE_UPY_IMAGE and config.USE_UPY_G11N:
    class ImageRepositoryG11nInline(G11nTabularInlineAdmin):
        """
        Admin option class for ImageRepositoryG11n inline
        """
        fieldsets = (('', {'fields': 
                           ('title','alt','description'),
            },),) + G11nAdmin.fieldsets
        model = ImageRepositoryG11n
    
    class ImageRepositoryG11nOption(G11nAdmin):
        """
        Concrete admin option class for ImageRepositoryG11n
        """
        list_display = ('id','title', 'alt', 'language')
        list_display_links = ['id']
        list_editable = ('title', 'alt', 'language',)
        list_filter = ('language',)
        fieldsets = (('', {'fields': 
                           ('title', 'alt', 'description','image_repository'),
            },),) + G11nAdmin.fieldsets
            
    class ImageRepositoryOption(PositionImageOption):
        """
        Concrete admin option class for ImageRepository
        """
        list_display = ('position','name','date','original_image','admin_thumbnail_view',)
        list_editable = ['position','date',]
        list_display_links = ['name']
        ordering = ('position',)
        save_on_top = True
        search_fields = ('name','date')
        inlines = [ImageRepositoryG11nInline,]
        form = UPYImageForm
    
    
    #admin.site.register(ImageRepositoryG11n, ImageRepositoryG11nOption)    
    admin.site.register(ImageRepository, ImageRepositoryOption)

elif config.USE_UPY_IMAGE:
    class ImageRepositoryOption(PositionImageOption):
        """
        Concrete admin option class for ImageRepository
        """
        list_display = ('position','name','date','title','alt','original_image','admin_thumbnail_view',)
        list_editable = ['position','date','title','alt']
        list_display_links = ['name']
        ordering = ('position',)
        save_on_top = True
        search_fields = ('name','title','alt','date')
        form = UPYImageForm
    
    admin.site.register(ImageRepository, ImageRepositoryOption)


from django.template.loader import render_to_string
from email.MIMEImage import MIMEImage
import mimetypes
from django.core.mail import EmailMessage, EmailMultiAlternatives
from django.template import loader, Context
from django.template import RequestContext
from django.utils.html import strip_tags

def umail(request, template, subject, sender, to, context = {}, bcc = [], attachments=[]):
    """
    It sends mail with template. It supports html and txt template. In the first case it searches a txt template with the
    same name in the same position. If it will be found it sends both.
    'template', 'subject' and 'sender' are strings.
    'to' and 'bcc' are lists of addresses.
    'context' is a dictionary.
    'attachments' is a list of paths.
    """
    if request:
        c = RequestContext(request, context)
    else:
        c = Context(context)

    if template.endswith('.html'):
        t = loader.get_template(template)
        try:
            t_html = t
            txt_name = '%s.txt' % (template.rsplit('.', 1)[0])
            t = loader.get_template(txt_name)
        except:
            print "Missing .txt template for: %s" % (template)
            email = EmailMessage(subject, t_html.render(c), sender, to, bcc)
            email.content_subtype = "html"
        else:
            email = EmailMultiAlternatives(subject, strip_tags(t.render(c)), sender, to, bcc)
            email.attach_alternative(t_html.render(c).encode('utf-8'), "text/html")
    else:
        t = loader.get_template(template)
        email = EmailMessage(subject, strip_tags(t.render(c)), sender, to, bcc)

        for filepath in attachments:
            mimetype = mimetypes.guess_type(filepath)[0]
            email.attach_file(filepath, mimetype)
    email.send()


def send_mail(subject, text_content, from_email, to, html_content = None, attachments = None, publication = ""):
    """
    This function sends mail using EmailMultiAlternatives and attachs all attachments
    passed as parameters
    """
    msg = EmailMultiAlternatives(subject, text_content, from_email, to)
    if html_content:
        msg.attach_alternative(html_content, "text/html")
    if attachments:
        for att in attachments:            
            mimetype = mimetypes.guess_type(att.file.url)[0]
            if str(mimetype) in ('image/jpeg', 'image/pjpeg', 'image/png', 'image/gif'):
                try:
                    with open(att.file.path) as f:
                        email_embed_image(msg, att.name, f.read())
                except Exception, e:
                    print e
            else:
                msg.attach_file("%s" % (att.file.url[1:]))
    msg.send()    
    
def send_rendered_mail(subject, text_content, template_name, context_dict, from_email,to, attachments = None):
    """
    It sends mail after rendering html content
    """
    rendered = render_to_string(template_name,context_dict)
    return send_mail(subject, text_content, from_email, to, rendered, attachments)


def email_embed_image(email, img_content_id, img_data):
    """
    email is a django.core.mail.EmailMessage object
    """
    img = MIMEImage(img_data)
    img.add_header('Content-ID', '<%s>' % img_content_id)
    img.add_header('Content-Disposition', 'inline')
    email.attach(img)
    


from django.contrib import admin
from upy.contrib.g11n.models import _,PublicationG11n,Publication,Language
from django.forms.models import BaseInlineFormSet
from django.conf import settings


class G11nAdmin(admin.ModelAdmin):
    """
    Base Admin for G11n Models
    """
    fieldsets = ((_(u"Globalization"),{
                    'fields':(('publication','language'),),
                    'classes': ('collapse',)}),)
        
    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "publication":
            if Publication.get_default():
                kwargs['initial'] = Publication.get_default().pk
        if db_field.name == "language":
            if Language.get_default():
                kwargs['initial'] = Language.get_default().pk
        return super(G11nAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)
      

class CyclePublication(object):
    """
    It returns a list of tuples with all combinations made with languages and publications. 
    All G11nModel classes inherit an admin's form with prepopulated language and publications.
    """
    positions = {}
    
    def __init__(self, is_publication = False):
        cont = 0
        if is_publication:
            for lang in Language.objects.all():
                self.positions['%s' % cont] = (None,lang)
                cont = cont + 1
        else:
            for publication in Publication.objects.all():
                for lang in publication.languages.all():
                    self.positions['%s' % cont] = (publication,lang)
                    cont = cont + 1
        

class G11nInlineFormset(BaseInlineFormSet):
    """
    This is the formset inherited in G11nInlineModel classes
    """
    def __init__(self, data=None, files=None, instance=None,
                 save_as_new=False, prefix=None, queryset=None):
        
        super(G11nInlineFormset, self).__init__(data, files, instance,
                 save_as_new, prefix, queryset) 
        cycle_publications = CyclePublication(instance.__class__.__name__ == "Publication")
        cont = 0
        if not data and cycle_publications.positions:
            for form in self.forms:
                (publication, lang) = cycle_publications.positions['%s' % cont]
                
                if instance.__class__.__name__ != "Publication": 
                    form.fields['publication'].initial = publication
                form.fields['language'].initial = lang
                cont  = cont + 1
                form.empty_permitted = True
                     
            
class G11nStackedInlineAdmin(admin.StackedInline):
    """
    It defines a StackedInlineAdmin for all G11nModelInline
    """
    def __init__(self, parent_model, admin_site):
        super(G11nStackedInlineAdmin,self).__init__(parent_model, admin_site)
        if self.parent_model._meta.object_name == "Publication":
            max_num = Language.objects.count()
        else:
            max_num = 0
            for publication in Publication.objects.all():
                max_num += publication.languages.all().count()
        self.max_num = max_num
        self.extra = max_num

        
    formset = G11nInlineFormset
    template = "admin/g11n_tabs_stacked.html"
    class Media: 
        css = {"all" : ("/upy_static/css/upy-stacked-tabs.css",)}
        js = (settings.JQUERY_LIB,
              '/upy_static/js/upy-admin-g11ninline.js',
              '/upy_static/js/upy-admin-g11ntabsinline.js',
              )
        
class G11nTabularInlineAdmin(admin.TabularInline):
    """
    It defines a TabularInlineAdmin for all G11nModelInline
    """
    def __init__(self, parent_model, admin_site):
        super(G11nTabularInlineAdmin,self).__init__(parent_model, admin_site)
        if self.parent_model._meta.object_name == "Publication":
            max_num = Language.objects.count()
        else:
            max_num = 0
            for publication in Publication.objects.all():
                max_num += publication.languages.all().count()
        self.max_num = max_num
        self.extra = max_num

    formset = G11nInlineFormset
    class Media: 
        js = ('/upy_static/js/upy-admin-g11ninline.js',)
    
class PublicationG11nInline(G11nStackedInlineAdmin):
    """
    It's admin's options for inline PublicationG11nModel 
    """
    model = PublicationG11n
    fieldsets = (('', {'fields': 
                       (('title', 'description'),
                        ('keywords','author'),
                        ('content_type','enabled'),
                        ('disabled_message',),
                       ('robots','generator')),
        },),) + G11nAdmin.fieldsets
        
class PublicationOption(admin.ModelAdmin): 
    """
    This is the option class for Publication Admin
    """
    list_display = ('name','url','is_default') 
    list_editable = ('url','is_default') 
    list_filter = ('name', 'languages',)
    fieldsets = (('',{'fields': (('name','url','favicon'),),}),
                 (_('Checking'),{'fields': (('enabled', 'is_default'),),}),
                 (_('Languages'),{'fields': (( 'languages','default_language',),),}),)
    
    inlines = [PublicationG11nInline,]
    save_on_top = True
    
    def save_model(self, request, obj, form, change):
        obj.url = obj.url.replace("http://","").replace("https://","")
        obj.save()
    
    class Meta: 
        model = Publication
    class Media: 
        js = (settings.JQUERY_LIB,
              '/upy_static/js/upy-admin-publication.js',)
        
class PublicationG11nOption(G11nAdmin): 
    """
    This is the option class for PublicationG11n Admin
    """
    list_display = ('title', 'author','enabled','publication','language') 
    list_filter = ('author', 'enabled',)
    save_on_top = True
    fieldsets = (('', {'fields': 
                       ('title', 'description','keywords','author',
                        'content_type','enabled','disabled_message',
                        'robots','generator',),
        },),) + G11nAdmin.fieldsets
    
    class Meta: 
        model = PublicationG11n
        
class LanguageOption(admin.ModelAdmin): 
    """
    This is the option class for Language Admin
    """
    list_display = ('name', 'code', 'alias',) 
    list_editable = ('code', 'alias',) 
    class Meta: 
        model = Language
        
admin.site.register(Publication, PublicationOption)
#admin.site.register(PublicationG11n, PublicationG11nOption)
admin.site.register(Language, LanguageOption)



"""
Newsletter contrib needs uwsgi application server to work because it uses the spooler decorator to send mails.
"""
from django.db import models,close_connection
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
import hmac
import hashlib
import base64
from django.conf import settings
from upy.utils import datetimeToUnixSec  
from upy.contrib.g11n.models import Publication
from django.db.models.signals import post_save
from datetime import datetime
try:
    from upy.uwsgidecorators import spool, timer
except Exception, e: 
    print "Warning in %s: %s" %(__file__,e)
from upy.contrib.newsletter.mail import send_mail

class UPYNL(models.Model):
    """
    It's an abstrat model used by all newsletter's model.
    It has only two fields that saves establishment datetime and last update datetime for each instance
    """
    creation_date = models.DateTimeField(auto_now_add = True, help_text = _(u"Establishment date"), 
                                        verbose_name = _(u"Creation date"))
    last_update = models.DateTimeField(auto_now = True, help_text = _(u"Last update"), 
                                        verbose_name = _(u"Last update"))
    class Meta:
        abstract = True

class Contact(UPYNL):
    """
    This represents a contact for lists of this newsletter
    """
    email = models.EmailField(help_text = _(u"Insert your e-mail."), verbose_name = _(u"E-mail"), unique = True)
    name = models.CharField(null = True, blank = True, max_length = 150, help_text = _(u"Insert you name"),
                            verbose_name = _(u"Name"))
    surname = models.CharField(null = True, blank = True, max_length = 150,
                                help_text = _(u"Insert you surname"), verbose_name = _(u"Surname"))
    secret_key = models.TextField(help_text = _(u"Set secret key"), verbose_name = _(u"Secret key"))
    confirmed = models.BooleanField(default = False, 
                                    help_text = _(u"Check if this contact is confirmed"), 
                                    verbose_name = _(u"Confirmed"))
    subscribed = models.BooleanField(default = True,
                                    help_text = _(u"Check if this contact is subscribed"), 
                                    verbose_name = _(u"Subscribed"))
        
    def __unicode__(self):
        if self.name and self.surname:
            return u"\"%s %s\" <%s> " % (self.surname, self.name, self.email)
        return u"%s" % (self.email)
    
    def save(self, *args, **kwargs):
        if not self.confirmed:
            tohash = "%s%s" % (self.email,datetime.now())
            dig = hmac.new(b'%s' % settings.UPY_SECRET_KEY, msg=tohash, digestmod=hashlib.sha256).digest()
            self.secret_key = base64.b64encode(dig).decode().replace("/","_")
        super(Contact,self).save( *args, **kwargs)
    
    class Meta:
        verbose_name = _(u"Contact")
        verbose_name_plural = _(u"Contacts")
        ordering = [u'email',u'surname',u'name']


class List(UPYNL):
    """
    A list groupes some contacts
    """
    name = models.CharField(max_length = 150, help_text = _(u"Insert list's name"), verbose_name = _(u"Name"))
    description = models.TextField(null = True, blank = True, help_text = _(u"Set list's description"), 
                                    verbose_name = _(u"Description"))
    priority = models.CharField(max_length = 150, help_text = _(u"Set list's priority"), 
                                    verbose_name = _(u"Priority"), choices = ([("%s"% n,n) for n in range(1,11)]))
    contacts = models.ManyToManyField(u"Contact", help_text = _(u"A list of contacts to associate"), 
                                    verbose_name = _(u"Contacts"))
    publication =models.ForeignKey(Publication,  help_text = _(u"Set the Publication for this list"), 
                                    verbose_name = _(u"Publication"))
    test = models.BooleanField(default = False, help_text = _(u"Check if this list is a test list"),
                                verbose_name = _(u"Test"))
       
    def __unicode__(self):
        return u"%s" % (self.name)
    
    class Meta:
        verbose_name = _(u"List")
        verbose_name_plural = _(u"Lists")
        ordering = [u'name']

class Attachment(UPYNL):
    """
    It represents a file to attach in a newsletter
    """
    name = models.CharField(max_length = 150, unique = True, help_text = _(u"Attachment's name"), 
                            verbose_name = _(u"Name"))
    attached_file = models.FileField(null = True, blank = True, upload_to = u"newsletter/attachments", 
                                        help_text = _(u"Upload the attachment's file."), verbose_name = _(u"File"))
    
    def __unicode__(self):
        return u"%s" % (self.name)
    
    class Meta:
        ordering = ['name']
        verbose_name = _(u"Attachment")
        verbose_name_plural = _(u"Attachments")
    
class Newsletter(UPYNL):
    """
    Here you can write your mail to send to some lists
    """
    subject = models.CharField(max_length = 250, help_text = _(u"Insert subject"), verbose_name = _(u"Subject"))
    body_text = models.TextField(help_text = _(u"Set body in text format"), verbose_name = _(u"Body text"))
    body_html = models.TextField(help_text = _(u"Set body in html format. If you want add an image in html content, you must replace url in src attribute with the cid content. Example src='cid:name_of_an_attachment_image'"), verbose_name = _(u"Body html"))
    attachments = models.ManyToManyField( u"Attachment", null = True, blank = True,
                                    help_text = _(u"A list of Attachments to associate"), 
                                    verbose_name = _(u"Attachments"))
    sent = models.BooleanField(default = False, 
                                help_text = _(u"Check if this newsletter is sent"),
                                verbose_name = _(u"Sent"))
    
    def __unicode__(self):
        return u"%s" % (self.subject)
    
    class Meta:
        verbose_name = _(u"Newsletter")
        verbose_name_plural = _(u"Newsletters")
        ordering = [u'subject']
        
class Dispatcher(UPYNL):
    """
    It's processed by spooler to send mails
    """
    contact_list = models.ForeignKey(u"List", help_text = _(u"Set a list of distribution"), verbose_name = _(u"List"))
    newsletter = models.OneToOneField(u"Newsletter", help_text = _(u"Set a newsletter"), verbose_name = _(u"Newsletter"), limit_choices_to = {'sent': False})
    last_mail_sent = models.ForeignKey(u"Contact", null = True, blank = True, help_text = _(u"Set the last mail sent"), verbose_name = _(u"Last mail sent"))
    status = models.CharField(max_length = 250, help_text = _(u"Set status"), verbose_name = _(u"Status"), default = u'waiting', choices = [(u'waiting',_(u'waiting')),
                                                                                                                                           (u'processing',_(u'processing')),
                                                                                                                                           (u'sent',_(u'sent')),
                                                                                                                                           (u'deleted',_(u'deleted')),
                                                                                                                                           (u'stopped',_(u'stopped')),
                                                                                                                                           ])
    send_date = models.DateTimeField(null = True, blank = True, help_text = _(u"Choose when to send the newsletter. Empty field means now"), verbose_name = _(u"Send date"))
    sent_date = models.DateTimeField(null = True, blank = True, help_text = _(u"Process complete date"), verbose_name = _(u"Sent date"))
    user_create = models.ForeignKey(User, null = True, blank = True, help_text = _(u"Set who create this dispatcher"), verbose_name = _(u"User create"))
    
    @property
    def priority(self):
        return self.contact_list.priority
    
    @property
    def current_status(self):
        disp = Dispatcher.objects.get(pk=self.pk)
        return disp.status
    
    def __unicode__(self):
        return u"%s - %s" % (self.newsletter, self.contact_list)
    
    class Meta:
        verbose_name = _(u"Dispatcher")
        verbose_name_plural = _(u"Dispatchers")
        ordering = [u'contact_list__priority',u'status']


def send_dispatcher_list(disp, recovery = False):
    """
    This function sends newsletter to all contacts in dispatcher's list
    """
    if recovery: lista = disp.contact_list.contacts.filter(email__gt=disp.last_mail_sent.email)
    else: lista = disp.contact_list.contacts.all()
    loop = 0
    for contact in lista:
        loop += 1
        if loop >= len(lista)/10: #ogni 10% dei contatti verifico se ho stoppato
            loop = 0
            if disp.current_status == "stopped": 
                return # se nel frattempo ho stoppato l'esecuzione esco
        contact_list = [contact.email] 
        # il campo email.to vuole una lista, ma noi mandiamo un indirizzo per volta per fare 
        #le email personalizzate e monitorare gli invii
        try:
            send_mail(disp.newsletter.subject,
                      disp.newsletter.body_text,
                      settings.EMAIL_USER,
                      contact_list,
                      disp.newsletter.body_html,
                      disp.newsletter.attachments.all(),
                      disp.contact_list.publication)
            
            disp.last_mail_sent = contact
            disp.save()
        except Exception, e:
            print e
            return
        
def close_dispatcher(disp):
    """
    utility function to set dispatcher and newsletter to sent (if not test list)
    """
    try:
        disp.status = "sent"
        disp.sent_date = datetime.now()
        if not disp.contact_list.test:
            newsl = disp.newsletter
            newsl.sent = True
        disp.save()
        newsl.save()
    except:
        raise

try:
    @spool
    def spooling_dispatcher (args):
        """
        This function enqueue a created dispatcher or a dispatcher with waiting status
        and which has not deleted status
        """
        try:
            close_connection()
        except:
            pass
        try:
            disp = Dispatcher.objects.get(pk=int(args['disp']))
            if disp.status == "deleted": return  
            # ho deciso di annullare l'invio dopo che il dispatcher era gia' stato spoolato. 
            # Return e bbona notte.
            disp.status = "processing"
            disp.save()
            try: 
                send_dispatcher_list(disp)
            except Exception, e:
                print e
                return #tanto lo status resta su processing, al prossimo giro del timer riprova a mandarlo
            else:    
                close_dispatcher(disp)
            
        except Exception, e:
            print e
    
    def run_dispatcher(sender, instance, created, **kwargs):
        """
        When you a dispatcher a new dispatcher, if it's new or its status is 'waiting', then 
        this dispather will be given to spooler that processes it
        """
        if created or instance.status=="waiting":  
            #se il dispatcher e' appena stato creato o ri-settato su waiting accodalo
            if not instance.send_date: instance.send_date = datetime.now()
            timestamp = datetimeToUnixSec(instance.send_date)
            spooling_dispatcher.spool(disp="%s" % instance.pk,at=timestamp)
    
    post_save.connect(run_dispatcher, sender=Dispatcher)
    
    
    @timer(settings.UPY_NEWSLETTER_SPOOLER_TIMEOUT, target='spooler')
    def check_dispatchers(args):
        """
        This function enqueue a processing dispatcher. It checks every UPY_NEWSLETTER_SPOOLER_TIMEOUT seconds.  
        """
        disps = Dispatcher.objects.filter(status='processing')
        for disp in disps:
            try: 
                send_dispatcher_list(disp, recovery=True)
            except Exception,e:
                print e
                return
            else:    
                close_dispatcher(disp)
except Exception, e:
    print "Warning in %s: %s" %(__file__,e)


import os
import unittest
from datetime import datetime
        
from django.conf import settings

from upy.contrib.ckeditor import views

class ViewsTestCase(unittest.TestCase):
    def setUp(self):
        # Retain original settings.
        self.orig_MEDIA_ROOT = settings.MEDIA_ROOT
        self.orig_CKEDITOR_UPLOAD_PATH = settings.CKEDITOR_UPLOAD_PATH
        self.orig_MEDIA_URL = settings.MEDIA_URL
        self.orig_CKEDITOR_RESTRICT_BY_USER = getattr(settings, 'CKEDITOR_RESTRICT_BY_USER', False)

        # Set some test settings.
        settings.MEDIA_ROOT = '/media/root/'
        settings.CKEDITOR_UPLOAD_PATH = os.path.join(settings.MEDIA_ROOT, 'uploads')
        settings.MEDIA_URL = '/media/'

        # Create dummy test upload path.
        self.test_path = os.path.join(settings.CKEDITOR_UPLOAD_PATH, 'arbitrary', 'path', 'and', 'filename.ext')
        
        # Create mock user.
        self.mock_user = type('User', (object,), dict(username='test_user', is_superuser=False))

    def tearDown(self):
        # Reset original settings.
        settings.MEDIA_ROOT = self.orig_MEDIA_ROOT
        settings.CKEDITOR_UPLOAD_PATH = self.orig_CKEDITOR_UPLOAD_PATH
        settings.MEDIA_URL = self.orig_MEDIA_URL
        settings.CKEDITOR_RESTRICT_BY_USER = self.orig_CKEDITOR_RESTRICT_BY_USER

    def test_get_media_url(self):
        # If provided prefix URL with CKEDITOR_UPLOAD_PREFIX.  
        settings.CKEDITOR_UPLOAD_PREFIX = '/media/ckuploads/'
        prefix_url = '/media/ckuploads/arbitrary/path/and/filename.ext'
        self.failUnless(views.get_media_url(self.test_path) == prefix_url)
        
        # If CKEDITOR_UPLOAD_PREFIX is not provided, the media URL will fall back to MEDIA_URL with the difference of MEDIA_ROOT and the uploaded resource's full path and filename appended.
        settings.CKEDITOR_UPLOAD_PREFIX = None
        no_prefix_url = '/media/uploads/arbitrary/path/and/filename.ext'
        self.failUnless(views.get_media_url(self.test_path) == no_prefix_url)
        
        # Resulting URL should never include '//'.
        self.failIf('//' in views.get_media_url(self.test_path))

    def test_get_thumb_filename(self):
        # Thumnbnail filename is the same as original with _thumb inserted before the extension. 
        self.failUnless(views.get_thumb_filename(self.test_path) == self.test_path.replace('.ext', '_thumb.ext'))
        # Without an extension thumnbnail filename is the same as original with _thumb appened.
        no_ext_path = self.test_path.replace('.ext', '')
        self.failUnless(views.get_thumb_filename(no_ext_path) == no_ext_path + '_thumb')
    
    def test_get_image_browse_urls(self):
        settings.MEDIA_ROOT = os.path.join(os.path.dirname(__file__), 'media')
        settings.CKEDITOR_UPLOAD_PATH = os.path.join(settings.MEDIA_ROOT, 'test_uploads')
        #settings.CKEDITOR_RESTRICT_BY_USER = True
        
        # The test_uploads path contains subfolders, we should eventually reach a single dummy resource. 
        self.failUnless(views.get_image_browse_urls())
        
        # Ignore thumbnails. 
        self.failUnless(len(views.get_image_browse_urls()) == 1)

        # Don't limit browse to user specific path if CKEDITOR_RESTRICT_BY_USER is False.
        settings.CKEDITOR_RESTRICT_BY_USER = False
        self.failUnless(len(views.get_image_browse_urls(self.mock_user)) == 1)
        
        # Don't limit browse to user specific path if CKEDITOR_RESTRICT_BY_USER is True but user is a superuser.
        settings.CKEDITOR_RESTRICT_BY_USER = True
        self.mock_user.is_superuser = True
        self.failUnless(len(views.get_image_browse_urls(self.mock_user)) == 1)
        
        # Limit browse to user specific path if CKEDITOR_RESTRICT_BY_USER is True and user is not a superuser.
        settings.CKEDITOR_RESTRICT_BY_USER = True
        self.mock_user.is_superuser = False
        self.failIf(views.get_image_browse_urls(self.mock_user))
        
        settings.CKEDITOR_RESTRICT_BY_USER = self.orig_CKEDITOR_RESTRICT_BY_USER

    def test_get_upload_filename(self):
        settings.CKEDITOR_UPLOAD_PATH = self.orig_CKEDITOR_UPLOAD_PATH
        date_path = datetime.now().strftime('%Y/%m/%d')
        
        # Don't upload to user specific path if CKEDITOR_RESTRICT_BY_USER is False.
        settings.CKEDITOR_RESTRICT_BY_USER = False
        filename = views.get_upload_filename('test.jpg', self.mock_user)
        self.failIf(filename.replace('/%s/test.jpg' % date_path, '').endswith(self.mock_user.username))
        
        # Upload to user specific path if CKEDITOR_RESTRICT_BY_USER is True.
        settings.CKEDITOR_RESTRICT_BY_USER = True
        filename = views.get_upload_filename('test.jpg', self.mock_user)
        self.failUnless(filename.replace('/%s/test.jpg' % date_path, '').endswith(self.mock_user.username))

        # Upload path should end in current date structure. 
        filename = views.get_upload_filename('test.jpg', self.mock_user)
        self.failUnless(filename.replace('/test.jpg', '').endswith(date_path))
        
        settings.CKEDITOR_RESTRICT_BY_USER = self.orig_CKEDITOR_RESTRICT_BY_USER



from django.contrib import admin
from django.conf import settings
from upy.contrib.g11n.admin import G11nAdmin,G11nStackedInlineAdmin
from upy.contrib.staticpage.models import StaticPage, StaticPageG11n, StaticPageCategory
from upy.contrib.ckeditor.widgets import CKEditorWidget
from django import forms

STATICPAGE_CKE_CONFIG = None
if settings.STATICPAGE_CKE_CONFIG:
    STATICPAGE_CKE_CONFIG = settings.STATICPAGE_CKE_CONFIG


class CKEditorForm(forms.ModelForm):
    """
    It's add a CKEditor widget to textarea fields
    """
    html = forms.CharField(
        widget=CKEditorWidget(config=STATICPAGE_CKE_CONFIG)
    )
    class Meta:
        model = StaticPageG11n

class StaticPageG11nOption(G11nAdmin):
    """
    Admin's options for StaticPageG11n model
    """
    list_display = ('id','alias', 'staticpage', 'language')
    list_display_links = ['id']
    list_editable = ('alias','language',)
    list_filter = ('language',)
    fieldsets = (('', {'fields': 
                       ('alias', 'html',),
        },),) + G11nAdmin.fieldsets
    form = CKEditorForm
    class Meta:
        model = StaticPageG11n

class StaticPageG11nInline(G11nStackedInlineAdmin):
    """
    Admin's options for StaticPageG11n model used inline
    """
    fieldsets = (('', {'fields': 
                       ('alias', 'html'),
        },),) + G11nAdmin.fieldsets
    model = StaticPageG11n
    form = CKEditorForm

class StaticPageOption(admin.ModelAdmin):
    """
    Admin's options for StaticPage model
    """
    
    list_display = ('page','category',)
    list_display_link = ('page',)
    list_editable = ('category',)
    list_filter = ('category',)
    inlines = [StaticPageG11nInline,]
    save_on_top = True
    class Meta:
        model = StaticPage

class StaticPageCategoryOption(admin.ModelAdmin):
    """
    Admin's options for StaticPageCategory model
    """
    list_display = ('name',)
    class Meta:
        model = StaticPageCategory 
        
        
admin.site.register(StaticPage, StaticPageOption)
#admin.site.register(StaticPageG11n, StaticPageG11nOption)
admin.site.register(StaticPageCategory, StaticPageCategoryOption)




from django.forms import ModelForm
from django.template.loader import render_to_string
from upy.contrib.newsletter.mail import send_rendered_mail
from django.utils.translation import ugettext_lazy as _
from upy.contrib.newsletter.models import Contact, List, settings
from datetime import datetime
from upy.contrib.g11n.models import get_current_publication

def upy_nl_confirm(request,secret_key, 
                         message_ok = _(u"Hi %s, your account is succesfully confirmed"), 
                         message_error = _(u"Some problems are occurred during confirmation. Please, contact %s"),
                         use_contact = True, email_host = settings.EMAIL_USER):
    """
    This view is used to confirm a registered Contact to a newsletter
    """
    try:
        contact = Contact.objects.get(secret_key = secret_key)
        contact.confirmed = True
        contact.save()
        if use_contact:
            message = message_ok.replace("%s","%s" % contact)
        else:
            message = message_ok.replace("%s","")
    except:
        contact = None
        message = message_error.replace("%s",email_host)
    
    return message

class UPYNewsletterForm(ModelForm):
    """
    It's represents a form with all fields if only_mail is True
    else it shows only email field
    """
    def __init__(self,only_email,*args, **kwargs):
        super(UPYNewsletterForm, self).__init__(*args, **kwargs)
        if only_email:
            exclude_list = ('name','surname','secret_key','confirmed','subscribed')
        else:
            exclude_list = ('secret_key','confirmed','subscribed')
        for field in exclude_list:
            del self.fields[field]
       
    class Meta:
        model = Contact
     
    def send_confirm_mail(self,request):
        """
        It sends a mail to a contact with confirmation link
        """
        current_publication = get_current_publication()
        try:
            contact = Contact.objects.get(email = request.POST.get("email"))
        except:
            contact = None
        if not contact:
            if self.is_valid():
                try:
                    contact = self.save()
                except:
                    return {"status": False, "error":"contact_save"}
                try:
                    nwl_list = List.objects.get(name = "List %s" % current_publication)
                except:
                    nwl_list = List(name = "List %s" % current_publication, 
                                                      description = "List created on %s" % datetime.now(),
                                                      priority = 1,
                                                      publication = current_publication)
                try:
                    nwl_list.save()
                    nwl_list.contacts.add(contact)
                    nwl_list.save()
                except:
                    result = {"status": False, "error":"list_save"}
                user = "%s" % contact
                
                confirm_url = "http://%s/confirm_subscription/%s" % (current_publication.url,contact.secret_key)
                message = render_to_string("send_confirmation.txt.html",{"user": user,
                                                                         "confirm_url":confirm_url,
                                                                         "website": "http://%s" % current_publication.url})
                send_rendered_mail(_(u"Confirm subscription"), message,
                                          "send_confirmation.html", {"user": user,
                                                                     "confirm_url":confirm_url,
                                                                     "website": "http://%s" % current_publication.url}, 
                                          settings.EMAIL_USER,[contact.email])
                result = {"status": True, "error":""}
                return result
            return {"status": False, "error":"form_not_valid"}
        else:
            return {"status": False, "error":"contact_exists"}
        
        



"""
Here there are some configurations for CKEditorWidget
"""
CKE_CONFIG_EASY = {
    #'contentsCss': '/static/admin/css/ckeditor.css',
    'height': 150,
    'width': 670,
    'forcePasteAsPlainText' : True,
    'toolbar': [['Bold', 'Italic', 'Underline', '-', 'TextColor', '-', 'Link', 'Unlink', '-', 'Source']],
}
"""
CKE_CONFIG_EASY is a basic configuration with following options: 'Bold', 'Italic', 'Underline', '-', 'TextColor', '-', 'Link', 'Unlink', '-', 'Source'
"""
CKE_CONFIG_ADVANCED = {
    'height': 250,
    'width': 670,
    'forcePasteAsPlainText' : True,
    'toolbar': [['Bold', 'Italic', 'Underline', 'Subscript', 'Superscript', '-', 'FontSize', 'TextColor', '-', 
                 'JustifyLeft','JustifyCenter','JustifyRight','JustifyBlock', '-', 'NumberedList', 'BulletedList', 
                 '-', 'Outdent', 'Indent', '-', 'Link', 'Unlink', '-', 'Source']],
}
"""
CKE_CONFIG_ADVANCED has more options to format text
"""